﻿using System;
using System.IO;
using System.Web.UI;
using System.Web.UI.Caching;
using System.Web.UI.WebControls;

[assembly: TagPrefix("Company.Web.WebControls", "CWeb")]
namespace Company.Web.WebControls
{

    /// <summary>
    /// Css include control
    /// </summary>
    [ToolboxData("<{0}:CSSInclude ID=\"CSSInclude1\" HRef=\"\" Rel=\"stylesheet\" Type=\"text/css\" CacheKey=\"CacheKey\" runat=\"server\"> </{0}:CSSInclude>")]
    public class CSSInclude : WebControl
    {

        #region Members

        private string _href = string.Empty;
        private string _rel = string.Empty;
        private string _type = string.Empty;
        private string _cacheKey = string.Empty;

        #endregion Members

        #region Properties

        /// <summary>
        /// Gets or sets the href
        /// </summary>
        public string HRef
        {
            get
            {
                return _href;
            }
            set
            {
                _href = value;
            }
        }

        /// <summary>
        /// Gets or sets the rel
        /// </summary>
        public string Rel
        {
            get
            {
                return _rel;
            }
            set
            {
                _rel = value;
            }
        }

        /// <summary>
        /// Gets or sets the type
        /// </summary>
        public string Type
        {
            get
            {
                return _type;
            }
            set
            {
                _type = value;
            }
        }

        /// <summary>
        /// Gets or sets the cache key
        /// </summary>
        public string CacheKey
        {
            get
            {
                return _cacheKey;
            }
            set
            {
                _cacheKey = value;
            }
        }


        #endregion Properties

        #region Event handlers

        /// <summary>
        /// Fires when the page loads
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // <link href=\"{0}\" rel=\"stylesheet\" type=\"text/css\">
            LiteralControl literal = new LiteralControl(string.Format("<link href=\"{0}\" rel=\"{1}\" type=\"{2}\">",
                GetIncludeUrlWithVersion(), Rel, Type));
            literal.EnableViewState = false;
            Controls.Add(literal);
        }

        #endregion Event handlers

        #region Methods

        /// <summary>
        /// Gets an includes url with it's version so that browser gets latest version on page refresh if the
        /// file has changed
        /// </summary>
        /// <returns></returns>
        private string GetIncludeUrlWithVersion()
        {
            string url = string.Empty;
            string version = "1";

            if (string.IsNullOrEmpty(CacheKey))
                throw new ArgumentException("The cacheKey must have a value.");

            if (Page.Cache[CacheKey] == null)
            {
                FileInfo fileInfo = new FileInfo(Page.Server.MapPath(HRef));

                if (fileInfo.Exists)
                {
                    version = fileInfo.LastWriteTime.ToString("ddMMyyyyhhmmss");
                    CacheDependency cacheDependency = new CacheDependency(fileInfo.FullName);
                    Page.Cache.Insert(CacheKey, version, cacheDependency);
                }
            }
            else
                version = Page.Cache[CacheKey] as string;

            return ResolveUrl(HRef) + "?v=" + version;
        }

        #endregion Methods

    }
}
